//=====================================================================
//	     .
//=====================================================================

//---------------------------------------------------------------------
#property copyright 	"Dima S., 2010."
#property link      	"dimascub@mail.com"
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//---------------------------------------------------------------------
enum ControlType
{
    STEP_WITH_HYSTERESISH = 0,									// Step With Hysteresis
    STEP_WITHOUT_HYSTERESIS = 1,								// Step Without Hysteresis
    LINEAR = 2,																	// Linear
    NON_LINEAR = 3,															// Nonlinear
};
//---------------------------------------------------------------------
//---------------------------------------------------------------------
enum LotsState
{
	LOTS_NORMAL = 1,																										//    
	LOTS_REJECTED = -1,																									//    
	LOTS_INTERMEDIATE = 0,																							//    
};
//---------------------------------------------------------------------

//=====================================================================
//	   :
//=====================================================================
class TradeSymbol
{
private:
	string		trade_symbol;																							//  

protected:
	double		min_trade_volume;																					//      
	double		max_trade_volume;																					//      
	double		min_trade_volume_step;																		//   
	double		max_total_volume;																					//     
	double		symbol_point;																							//   
	double		symbol_point_multiplier;																	//   
	double		symbol_tick_size;																					//   
	int				symbol_digits;																						//    
	datetime	symbol_time;																							//    
	double		symbol_stop_level;																				//     (  )
	double		symbol_spread;																						// 
	double		symbol_bid;
	double		symbol_ask;
 
public:
	void			RefreshSymbolInfo( );																			//      
	void			SetTradeSymbol( string _symbol );													// /  
	string		GetTradeSymbol( );																				//   
	double		GetMaxTotalLots( );																				//    
	double		GetMinTradeVolume( );																			//       
	double		GetMaxTradeVolume( );																			//       
	double		GetPoints( double _delta );																//     
	double		GetDelta( double _points );																//      
	datetime	GetSymbolTime( );																					//     

public:
	double		NormalizeLots( double _requied_lot );											//    
	double		NormalizePrice( double _org_price );											//        
	double		NormalizeStopLoss( double _stop_loss, long _type );				//   
	double		GetStopLoss( double _points, long _type );								//     ,   
	double		GetTakeProfit( double _points, long _type );							//     ,   

public:
	void			TradeSymbol( );																						// 
	void			~TradeSymbol( );																					// 
};

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TradeSymbol::TradeSymbol( )
{
	this.trade_symbol = NULL;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TradeSymbol::~TradeSymbol( )
{
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
void
TradeSymbol::RefreshSymbolInfo( )
{
//	    ,     :
	if( this.GetTradeSymbol( ) == NULL )
	{
		return;
	}

//	 ,    :
	this.min_trade_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_MIN );
	this.max_trade_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_MAX );
	this.min_trade_volume_step = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_STEP );

	this.max_total_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_LIMIT );

	this.symbol_point = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_POINT );
	this.symbol_tick_size = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_TRADE_TICK_SIZE );
	this.symbol_digits = ( int )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_DIGITS );

	this.symbol_stop_level = ( double )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_TRADE_STOPS_LEVEL );
	this.symbol_spread = ( double )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_SPREAD );
	this.symbol_bid = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_BID );
	this.symbol_ask = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_ASK );

}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
string
TradeSymbol::GetTradeSymbol( )
{
	return( this.trade_symbol );
}

//---------------------------------------------------------------------
//	/  :
//---------------------------------------------------------------------
void
TradeSymbol::SetTradeSymbol( string _symbol )
{
	this.trade_symbol = _symbol;
	this.RefreshSymbolInfo( );
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double
TradeSymbol::GetMaxTotalLots( )
{
	return( this.max_total_volume );
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
double
TradeSymbol::GetMinTradeVolume( )
{
	return( this.min_trade_volume );
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
double
TradeSymbol::GetMaxTradeVolume( )
{
	return( this.max_trade_volume );
}

//---------------------------------------------------------------------
//	    : 
//---------------------------------------------------------------------
datetime
TradeSymbol::GetSymbolTime( )
{
	this.symbol_time = ( datetime )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_TIME );
	return( this.symbol_time );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
double
TradeSymbol::GetPoints( double _delta )
{
	return( NormalizeDouble( _delta / this.symbol_point, 0 ));
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
double
TradeSymbol::GetDelta( double _points )
{
	return( NormalizeDouble( _points * this.symbol_point, this.symbol_digits ));
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
double
TradeSymbol::GetStopLoss( double _points, long _type )
{
	if( _type == ( long )POSITION_TYPE_BUY )
	{
		return( NormalizeStopLoss( this.symbol_bid - _points * this.symbol_point, _type ));
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
		return( NormalizeStopLoss( this.symbol_ask + _points * this.symbol_point, _type ));
	}

	return( 0.0 );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
double
TradeSymbol::GetTakeProfit( double _points, long _type )
{
	if( _type == ( long )POSITION_TYPE_BUY )
	{
		return( NormalizeDouble( this.symbol_ask + _points * this.symbol_point, this.symbol_digits ));
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
			return( NormalizeDouble( this.symbol_bid - _points * this.symbol_point, this.symbol_digits ));
	}

	return( 0.0 );
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
//	-    ;
//	-    ;
//---------------------------------------------------------------------
double
TradeSymbol::NormalizeLots( double _requied_lots )
{
	double	lots, koeff;
	int			nmbr;

//	    ,     :
	if( this.GetTradeSymbol( ) == NULL )
	{
		return( 0.0 );
	}

	if( this.min_trade_volume_step > 0.0 )
	{
		koeff = 1.0 / this.min_trade_volume_step;
		nmbr = ( int )MathLog10( koeff );
	}
	else
	{
		koeff = 1.0 / this.min_trade_volume;
		nmbr = 2;
	}
	lots = MathFloor( _requied_lots * koeff ) / koeff;

//	  :
	if( lots < this.min_trade_volume )
	{
		lots = this.min_trade_volume;
	}

//	  :
	if( lots > this.max_trade_volume )
	{
		lots = this.max_trade_volume;
	}

	lots = NormalizeDouble( lots, nmbr );
	return( lots );
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
double
TradeSymbol::NormalizePrice( double _org_price )
{
//	    , :
	double	min_price_step = NormalizeDouble( this.symbol_tick_size / this.symbol_point, 0 );

	double	norm_price = NormalizeDouble( NormalizeDouble(( NormalizeDouble( _org_price / this.symbol_point, 0 )) / min_price_step, 0 ) * min_price_step * this.symbol_point, this.symbol_digits );
	return( norm_price );
}

//---------------------------------------------------------------------
//	    /  :
//---------------------------------------------------------------------
//	-      /;
//	-   / ,     ;
//	-   /     ;
//---------------------------------------------------------------------
double
TradeSymbol::NormalizeStopLoss( double _stop_loss, long _type )
{
	double	corrected_stop = _stop_loss;
	double	delta = NormalizeDouble(( this.symbol_stop_level + 2.0 * this.symbol_spread ) * this.symbol_point, this.symbol_digits );

	if( _type == ( long )POSITION_TYPE_BUY )
	{
//	,          (     ):
		if( /*MathAbs*/( this.symbol_bid - corrected_stop ) <= delta )
		{
			corrected_stop = this.symbol_bid - delta;			//     
		}
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
//	,          (     ):
		if( /*MathAbs*/( corrected_stop - this.symbol_ask ) <= delta )
		{
			corrected_stop = this.symbol_ask + delta;			//     
		}
	}

	return( this.NormalizePrice( corrected_stop ));
}

//=====================================================================
//	   :
//=====================================================================
class TBalanceHistory
{
private:
	long			current_magic;																						//  " "      ( 0 -   )
	long			current_type;																							//   ( -1 -  )
	int				current_limit_history;																		//     ( 0 -   )
	datetime	monitoring_begin_date;																		//     
	int				real_trades;																							//     

public:
	TradeSymbol	trade_symbol;																						//    

protected:
//	"" :
	double		org_datetime_array[ ];																		// / 
	double		org_result_array[ ];																			//  

//	     :
	double		group_datetime_array[ ];																	// / 
	double		group_result_array[ ];																		//  

	double		last_result_array[ ];																			//       (    OY )
	double		last_datetime_array[ ];																		//       (    OX )

private:
	void			SortMasterSlaveArray( double& _m[ ], double& _s[ ] );			//      

public:
	void			SetTradeSymbol( string _symbol );													// /  
	string		GetTradeSymbol( );																				//   
	void			RefreshSymbolInfo( );																			//      
	void			SetMonitoringBeginDate( datetime _dt );										//    
	datetime	GetMonitoringBeginDate( );																//    
	int				GetRealTrades( );																					//    
	void			SetFiltrParams( long _magic, long _type = -1, int _limit = 0 );//    

public:
	int				GetTradeResultsArray( int _max_trades );									//    

public:
	void			TBalanceHistory( );																				// 
	void			~TBalanceHistory( );																			// 
};

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceHistory::TBalanceHistory( )
{
	this.current_magic = 0;
	this.current_type = -1;
	this.current_limit_history = 0;
	this.monitoring_begin_date = 0;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceHistory::~TBalanceHistory( )
{
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
int
TBalanceHistory::GetRealTrades( )
{
	return( this.real_trades );
}

//---------------------------------------------------------------------
//	/  :
//---------------------------------------------------------------------
void
TBalanceHistory::SetTradeSymbol( string _symbol )
{
	this.trade_symbol.SetTradeSymbol( _symbol );
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
string
TBalanceHistory::GetTradeSymbol( )
{
	return( this.trade_symbol.GetTradeSymbol( ));
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
void
TBalanceHistory::RefreshSymbolInfo( )
{
	this.trade_symbol.RefreshSymbolInfo( );
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
void
TBalanceHistory::SetMonitoringBeginDate( datetime _dt )
{
	this.monitoring_begin_date = _dt;
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
void
TBalanceHistory::SetFiltrParams( long _magic, long _type, int _limit )
{
	this.current_magic = _magic;
	this.current_type = _type;
	this.current_limit_history = _limit;
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
datetime
TBalanceHistory::GetMonitoringBeginDate( )
{
	return( this.monitoring_begin_date );
}

//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
void
TBalanceHistory::SortMasterSlaveArray( double& _master[ ], double& _slave[ ] )
{
	int				size = ArraySize( _master );
	if( size != ArraySize( _slave ))
	{
		return;
	}

	double		temp_m = 0.0;
	double		temp_s;

	for( int i = 1;  i <= size; i++ )
	{
		for( int j = 0;  j < size - i;  j++ )
		{
//	    ,    :
			if( _master[ j ] < _master[ j + 1 ] )
			{
				temp_m = _master[ j ]; 
				_master[ j ] = _master[ j + 1 ];
				_master[ j + 1 ] = temp_m;

				temp_s = _slave[ j ]; 
				_slave[ j ] = _slave[ j + 1 ];
				_slave[ j + 1 ] = temp_s;
			}
		}
	}
}

//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
//	-     ,    ;
//---------------------------------------------------------------------
int
TBalanceHistory::GetTradeResultsArray( int _max_trades )
{
	int				index, limit, count;
	long			deal_type, deal_magic, deal_entry;
	int				deal_close_time, current_time;
	ulong			deal_ticket;																						//  
	double		trade_result;
	string		symbol, deal_symbol;

	this.real_trades = 0;

//	      :
	if( _max_trades < 2 )
	{
		return( 0 );
	}

//	    ,     :
	symbol = this.trade_symbol.GetTradeSymbol( );
	if( symbol == NULL )
	{
		return( 0 );
	}

//	     c     :
	if( HistorySelect( this.monitoring_begin_date, TimeCurrent( )) != true )
	{
		return( 0 );
	}

//	  :
	count = HistoryDealsTotal( );

//	    ,  ,    :
	if( count < _max_trades )
	{
		return( 0 );
	}

//	    ,  ,  :
	if( this.current_limit_history > 0 && count > this.current_limit_history )
	{
		limit = count - this.current_limit_history;
	}
	else
	{
		limit = 0;
	}

//	 ,   ""     :
	if(( ArraySize( this.org_datetime_array )) != ( count - limit ))
	{
		ArrayResize( this.org_datetime_array, count - limit );
		ArrayResize( this.org_result_array, count - limit );
	}

//	 ""     :
	for( index = count - 1; index >= limit; index-- )
	{
		deal_ticket = HistoryDealGetTicket( index );

//	    ,    :
		deal_entry = HistoryDealGetInteger( deal_ticket, DEAL_ENTRY );
		if( deal_entry != DEAL_ENTRY_OUT )
		{
			continue;
		}

//	 " " ,  :
		deal_magic = HistoryDealGetInteger( deal_ticket, DEAL_MAGIC );
		if( this.current_magic != 0 && deal_magic != this.current_magic )
		{
			continue;
		}

//	  :
		deal_symbol = HistoryDealGetString( deal_ticket, DEAL_SYMBOL );
		if( symbol != deal_symbol )
		{
			continue;
		}
		
//	  ,  :
		deal_type = HistoryDealGetInteger( deal_ticket, DEAL_TYPE );
		if( this.current_type != -1 && deal_type != this.current_type )
		{
			continue;
		}
		else if( this.current_type == -1 && ( deal_type != DEAL_TYPE_BUY && deal_type != DEAL_TYPE_SELL ))
		{
			continue;
		}
		
//	   :
		deal_close_time = ( int )HistoryDealGetInteger( deal_ticket, DEAL_TIME );
		if( deal_close_time < ( int )this.monitoring_begin_date )
		{
			continue;
		}

//	,    :
		this.org_datetime_array[ this.real_trades ] = ( double )( deal_close_time / 60 );
		this.org_result_array[ this.real_trades ] = HistoryDealGetDouble( deal_ticket, DEAL_PROFIT ) / HistoryDealGetDouble( deal_ticket, DEAL_VOLUME );
		this.real_trades++;
	}

//	  ,  ,    :
	if( this.real_trades < _max_trades )
	{
		return( 0 );
	}
	count = this.real_trades;

//	 ""   /  :
	SortMasterSlaveArray( this.org_datetime_array, this.org_result_array );

//	 ,        :
	if(( ArraySize( this.group_datetime_array )) != count )
	{
		ArrayResize( this.group_datetime_array, count );
		ArrayResize( this.group_result_array, count );
	}
	ArrayInitialize( this.group_datetime_array, 0.0 );
	ArrayInitialize( this.group_result_array, 0.0 );

//	     (    /   ):
	this.real_trades = 0;
	for( index = 0; index < count; index++ )
	{
//	  :
		deal_close_time = ( int )this.org_datetime_array[ index ];
		trade_result = this.org_result_array[ index ];

//	 ,        :
		current_time = ( int )this.group_datetime_array[ this.real_trades ];
		if( current_time > 0 && ( int )MathAbs( current_time - deal_close_time ) > 0 )
		{
			this.real_trades++;												//     
			this.group_result_array[ this.real_trades ] = trade_result;
			this.group_datetime_array[ this.real_trades ] = deal_close_time;
		}
		else
		{
			this.group_result_array[ this.real_trades ] += trade_result;
			this.group_datetime_array[ this.real_trades ] = deal_close_time;
		}
	}
	this.real_trades++;														//     

//	  ,  ,    :
	if( this.real_trades < _max_trades )
	{
		return( 0 );
	}

	if( ArraySize( this.last_result_array ) != _max_trades )
	{
		ArrayResize( this.last_result_array, _max_trades );
		ArrayResize( this.last_datetime_array, _max_trades );
	}

//	        :
	for( index = 0; index < _max_trades; index++ )
	{
		this.last_result_array[ _max_trades - 1 - index ] = this.group_result_array[ index ];
		this.last_datetime_array[ _max_trades - 1 - index ] = this.group_datetime_array[ index ];
	}

//	         :
	for( index = 1; index < _max_trades; index++ )
	{
		this.last_result_array[ index ] += this.last_result_array[ index - 1 ];
	}

	return( _max_trades );
}


//=====================================================================
//	   :
//=====================================================================
class TBalanceSlope : public TBalanceHistory
{
protected:
	double	current_slope;																								//     

private:
	int			slope_count_points;																						//   (  )    
	
private:
	double	LR_koeff_A, LR_koeff_B;																				//      
	double	LR_points_array[ ];																						//     

private:
	void		CalcLR( double& X[ ], double& Y[ ] );													//     

public:
	void		SetSlopePoints( int _number );																//       
	int			GetSlopePoints( );																						//       
	double	CalcSlope( );																									//   
	double	GetCurrentSlope( );																						//    

public:
	void		TBalanceSlope( );																							// 
	void		~TBalanceSlope( );																						// 
};

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceSlope::TBalanceSlope( )
{
	this.slope_count_points = 3;
	this.current_slope = 0.0;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceSlope::~TBalanceSlope( )
{
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
void
TBalanceSlope::SetSlopePoints( int _number )
{
	this.slope_count_points = _number;
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
int
TBalanceSlope::GetSlopePoints( )
{
	return( this.slope_count_points );
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double
TBalanceSlope::GetCurrentSlope( )
{
	return( this.current_slope );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
//	:   aka Prival,  Skype: privalov-sv
//---------------------------------------------------------------------
//	 :
//		X[ ] -       X;
//		Y[ ] -       Y;
//---------------------------------------------------------------------
void
TBalanceSlope::CalcLR( double& X[ ], double& Y[ ] )
{
	double		mo_X = 0, mo_Y = 0, var_0 = 0, var_1 = 0;
	int				i;
	int				size = ArraySize( X );
	double		nmb = ( double )size;

//	    ,    :
	if( size < 2 )
	{
		return;
	}

	for( i = 0; i < size; i++ )
	{
		mo_X += X[ i ];
		mo_Y += Y[ i ];
	}
	mo_X /= nmb;
	mo_Y /= nmb;

	for( i = 0; i < size; i++ )
	{
		var_0 += ( X[ i ] - mo_X ) * ( Y[ i ] - mo_Y );
		var_1 += ( X[ i ] - mo_X ) * ( X[ i ] - mo_X );
	}

//	  A:
	if( var_1 != 0.0 )
	{
		this.LR_koeff_A = var_0 / var_1;
	}
	else
	{
		this.LR_koeff_A = 0.0;
	}

//	  B:
	this.LR_koeff_B = mo_Y - this.LR_koeff_A * mo_X;

//	  ,    :
	ArrayResize( this.LR_points_array, size );
	for( i = 0; i < size; i++ )
	{
		this.LR_points_array[ i ] = this.LR_koeff_A * X[ i ] + this.LR_koeff_B;
	}
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
double
TBalanceSlope::CalcSlope( )
{
//	      :
	int			nmb = this.GetTradeResultsArray( this.slope_count_points );
	if( nmb < this.slope_count_points )
	{
		return( 0.0 );
	}

//	      :
	this.CalcLR( last_datetime_array, last_result_array );
	this.current_slope = this.LR_koeff_A;

	return( this.current_slope );
}


//=====================================================================
//	   :
//=====================================================================
class TBalanceSlopeControl : public TBalanceSlope
{
private:
	double	min_slope;																										//  ,    
	double	max_slope;																										//  ,    
	double	centr_slope;																									//  ,     

private:
	ControlType	control_type;																							//   

private:
	double	rejected_lots;																								//     
	double	normal_lots;																									//     
	double	intermed_lots;																								//     

private:
	LotsState		current_lots_state;																				//   

public:
	void		SetControlType( ControlType _control );												//    
	void		SetControlParams( double _min_slope, double _max_slope, double _centr_slope );

public:
	double	CalcTradeLots( double _min_lots, double _max_lots );					//   

protected:
	double	CalcIntermediateLots( double _min_lots, double _max_lots, double _slope );

public:
	void		TBalanceSlopeControl( );																			// 
	void		~TBalanceSlopeControl( );																			// 
};

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceSlopeControl::TBalanceSlopeControl( )
{
	this.current_lots_state = LOTS_REJECTED;															//       
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void
TBalanceSlopeControl::~TBalanceSlopeControl( )
{
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
void
TBalanceSlopeControl::SetControlType( ControlType _control )
{
	this.control_type = _control;
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
void
TBalanceSlopeControl::SetControlParams( double _min_slope, double _max_slope, double _centr_slope )
{
	this.min_slope = _min_slope;																			//  ,    
	this.max_slope = _max_slope;																			//  ,    
	this.centr_slope = _centr_slope;																	//  ,     
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
double
TBalanceSlopeControl::CalcTradeLots( double _min_lots, double _max_lots )
{
//	    :
	this.CalcSlope( );

//	      ,    :
	if( this.GetRealTrades( ) < this.GetSlopePoints( ))
	{
		this.current_lots_state = LOTS_REJECTED;
		this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
		return( this.rejected_lots );
	}

//	     :
	if( this.control_type == STEP_WITHOUT_HYSTERESIS )
	{
		if( current_slope < this.centr_slope )
		{
			this.current_lots_state = LOTS_REJECTED;
			this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
			return( this.rejected_lots );
		}
		else
		{
			this.current_lots_state = LOTS_NORMAL;
			this.normal_lots = this.trade_symbol.NormalizeLots( _max_lots );
			return( this.normal_lots );
		}
	}

//	        :
	if( current_slope < this.min_slope )
	{
		this.current_lots_state = LOTS_REJECTED;
		this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
		return( this.rejected_lots );
	}

//	       :
	if( current_slope > this.max_slope )
	{
		this.current_lots_state = LOTS_NORMAL;
		this.normal_lots = this.trade_symbol.NormalizeLots( _max_lots );
		return( this.normal_lots );
	}

//	         (   ):
	this.current_lots_state = LOTS_INTERMEDIATE;

//	   :
	this.intermed_lots = CalcIntermediateLots( _min_lots, _max_lots, current_slope );
	this.intermed_lots = this.trade_symbol.NormalizeLots( this.intermed_lots );

	return( this.intermed_lots );
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
double
TBalanceSlopeControl::CalcIntermediateLots( double _min_lots, double _max_lots, double _slope )
{
	double		lots = _min_lots;

//	     :
	if( this.control_type == STEP_WITH_HYSTERESISH )
	{
		if( this.current_lots_state == LOTS_REJECTED && _slope > this.min_slope && _slope < this.max_slope )
		{
			lots = _min_lots;
		}
		else if( this.current_lots_state == LOTS_NORMAL && _slope > this.min_slope && _slope < this.max_slope )
		{
			lots = _max_lots;
		}
	}
//	   :
	else if( this.control_type == LINEAR )
	{
		double	a = ( _max_lots - _min_lots ) / ( this.max_slope - this.min_slope );
		double	b = normal_lots - a * this.max_slope;
		lots = a * _slope + b;
	}
//	     (    ):
	else if( this.control_type == NON_LINEAR )
	{
		lots = _min_lots;
	}
//	    :
	else
	{
		lots = _min_lots;
	}

	return( lots );
}
//---------------------------------------------------------------------
